from IPython.display import Javascript, display

from .tokenizer import tokenize
from .wordcount import wordcount
from .corpus import corpus
from .grams import grams
from .smoothing import smoothing
from .pos import pos
from .stopwords import stopwords
from .stemming import stemming
from .constiuency import constiuency
from .dependency import dependency
from .rule import rule
from .polarity import polarity

# from .slr import slr
# from .mlr import mlr
# from .blgr import blgr
# from .mlgr import mlgr
# from .dt import dt
# from .gd import gd
# from .km import km
# from .knn import knn
# from .ar import ar
# from .nb import nb




def print_codes(codes):
    codes = codes[::-1]

    for code in codes:
        js_code = f'''
        var cell = Jupyter.notebook.insert_cell_below('code');
        cell.set_text(`{code}`);
        '''
        display(Javascript(js_code))



# def code(program):
#     codes = []
#     if program == "19-12-18":
#         codes = slr()
#     elif program == "13-12-18":
#         codes = mlr()
#     elif program == "2-12-7-18":
#         codes = blgr()
#     elif program == "13-12-7-18":
#         codes = mlgr()
#     elif program == "4-20":
#         codes = dt()
#     elif program == "7-4":
#         codes = gd()
#     elif program == "11-13":
#         codes = km()
#     elif program == "11-14-14":
#         codes = knn()
#     elif program == "1-18":
#         codes = ar()
#     elif program == "14-2":
#         codes = nb()
#     elif program == "diddy":
#         codes = [
#             """
#             SLR = 19-12-18

#             MLR = 13-12-18

#             BLGR = 2-12-7-18

#             MLGR = 13-12-7-18

#             DT = 4-20

#             GD = 7-4

#             KM = 11-13

#             KNN = 11-14-14

#             AR = 1-18

#             NB = 14-2
#             """
#         ]
#     else:
#         codes = ["Nice Try Diddy"]
#     print_codes(codes)


def code(program):
    if (1 == 1):
        codes = []
    elif program == "tokenize":
        codes = tokenize()
    elif program == "wordcount":
        codes = wordcount()
    elif program == "corpus":
        codes = corpus()
    elif program == "grams":
        codes = grams()
    elif program == "smoothing":
        codes = smoothing()
    elif program == "pos":
        codes = pos()
    elif program == "stopwords":
        codes = stopwords()
    elif program == "stemming":
        codes = stemming()
    elif program == "constiuency":
        codes = constiuency()
    elif program == "dependency":
        codes = dependency()
    elif program == "rule":
        codes = rule()
    elif program == 'polarity':
        codes = polarity()
    elif program == "capy":
        codes = [
            """tokenize, wordcount, corpus, grams, smoothing, pos, stopwords, stemming, constiuency, dependency, rule, polarity"""
        ]
    else:
        codes = ["""Nice Try Diddy"""]

    print_codes(codes)


def dependency():
    return [
        """
        import spacy
        from spacy import displacy
        """,
        """
        nlp = spacy.load("en_core_web_sm")
        """,
        """
        def dependency(sentence):
            doc = nlp(sentence)
            return doc
        """,
        """
        sentence = "The cat sat on the mat"
        """,
        """
        doc = dependency(sentence)
        """,
        """
        print(f"sentence: {sentence}")
        """,
        """
        for token in doc:
            print(f"{token.text:10} {token.dep_:10} {token.head.text:10}")
        """,
        """
        displacy.serve(doc, style="dep")
        """
    ]


def wordcount():
    return [
        """
        import nltk
        from nltk.tokenize import word_tokenize
        from collections import Counter

        nltk.download('punkt')
        """,
        """
        def analyze_sentence(sentence, top_n):
            words = word_tokenize(sentence.lower())
            words = [word for word in words if word.isalnum()]
            word_counts = Counter(words)
            print("Word Counts:")
            for word, count in word_counts.items():
                print(f"{word}: {count}")
            print("\nTop", top_n, "words:")
            for word, count in word_counts.most_common(top_n):
                print(f"{word}: {count}")
        """,
        """
        sentence = "The quick brown fox jumps over the lazy dog. The fox is quick."
        top_n = 3
        analyze_sentence(sentence, top_n)
        """
    ]



def grams():
    return [
        """
        import nltk
        from nltk.util import ngrams
        from nltk.tokenize import word_tokenize
        from collections import Counter
        """,
        """
        corpus = [
            "I skipped my breakfast today",
            "I ate my breakfast today",
            "I ate my lunch yesterday",
            "I skipped my lunch today"
        ]
        """,
        """
        tokenized_corpus = []
        for sentence in corpus:
            var = sentence.lower()
            var = word_tokenize(sentence)
            tokenized_corpus.append(var)
        """,
        """
        corpus_words = []
        for i in tokenized_corpus:
            for word in i:
                corpus_words.append(word)
        """,
        """
        vocab = set(corpus_words)
        """,
        """
        def grams(n):
            result = []
            for i in range(len(corpus_words)-n+1):
                result.append(tuple(corpus_words[i:i+n]))
            return result
        """,
        """
        print(grams(1))
        """,
        """
        print(grams(2))
        """,
        """
        print(grams(3))
        """
    ]


def polarity():
    return [
        """
        from textblob import TextBlob

        sentence = "The quick brown fox jumps over the lazy dog."

        blob = TextBlob(sentence)

        polarity = blob.sentiment.polarity

        if polarity > 0:
            print("Positive Sentiment")
        elif polarity < 0:
            print("Negative Sentiment")
        else:
            print("Neutral Sentiment")

        print(f"Polarity Score: {polarity :.2f}")
        """
    ]


def tokenize():
    return [
        """
        import nltk
        nltk.download('punkt')
        nltk.download('averaged_perceptron_tagger')
        """,
        """
        import nltk
        nltk.download('punkt')
        nltk.download('averaged_perceptron_tagger')
        """,
        """
        from nltk.tokenize import sent_tokenize, word_tokenize
        from nltk import pos_tag

        def clear(file):

            sentences = sent_tokenize(file)
            words = word_tokenize(file)    
            words = [word.lower() for word in words if word.isalpha()]
            tags = pos_tag(words)

            print(f"Length of sentence: {len(sentences)}")
            for i, sentence in enumerate(sentences, start=1): 
                print(f"{i}: {sentence}")

            print(f"\nToatal number of words: {len(words)} \nWords: {words}\n")

            unique = set(words)
            print(f"Total Distinct words: {len(unique)}\n {unique}\n")

        clear(file)
        """
        
    ]


def corpus():
    return [
        """
        import nltk
        from nltk.tokenize import word_tokenize
        nltk.download('punkt', quiet=True)
        """,
        """
        dummy_corpus = [
            "This is the first document.",
            "This document is the second document.",
            "And this is the third one.",
            "Is this the first document?",
            "This is another document."
        ]
        """,
        """
        def analyze_corpus(corpus):
            corpus_words = []
            for document in corpus:
                words = word_tokenize(document.lower())
                words = [word for word in words if word.isalnum()]
                corpus_words.extend(words)
            vocabulary = sorted(set(corpus_words))
            print("Corpus Words:")
            print(corpus_words)
            print("\nVocabulary:")
            print(vocabulary)
        """,
        """
        analyze_corpus(dummy_corpus)
        """
    ]


def stemming():
    return [
        """
        import nltk
        from nltk.tokenize import word_tokenize
        from nltk.stem import PorterStemmer

        nltk.download('punkt', quiet=True)
        """,
        """
        def stemming_corpus(corpus):
            stemmer = PorterStemmer()
            stemmed_corpus = []

            for document in corpus:
                words = word_tokenize(document.lower())
                stemmed_words = [stemmer.stem(word) for word in words if word.isalnum()]
                stemmed_corpus.append(" ".join(stemmed_words))

            return stemmed_corpus
        """,
        """
        dummy_corpus = [
            "This is the first document.",
            "This document is the second document.",
            "And this is the third one.",
            "Is this the first document?",
            "This is another document."
        ]
        """,
        """
        stemmed_corpus = stemming_corpus(dummy_corpus)
        print(stemmed_corpus)
        """
    ]


def constiuency():
    return [
        """
        import stanza
        import nltk
        from nltk.tree import Tree
        """,
        """
        nlp = stanza.Pipeline('en')
        """,
        """
        # Example sentence
        sentence = "The cat sat on the mat."

        # Process the sentence using the stanza pipeline
        doc = nlp(sentence)
        """,
        """
        # Get the constituency tree
        for sentence in doc.sentences:
            constituency_tree = sentence.constituency

            # Convert the string representation to a Tree object
            tree = Tree.fromstring(str(constituency_tree))

            # Display the tree
            tree.pretty_print()
        """
    ]


def pos():
    return [
        """
        import nltk
        from nltk.tokenize import word_tokenize

        nltk.download('punkt', quiet=True)
        nltk.download('averaged_perceptron_tagger', quiet=True)
        """,
        """
        def pos_tagging(sentence):
            words = word_tokenize(sentence)
            tagged_words = nltk.pos_tag(words)
            return tagged_words
        """,
        """
        sentence = "The quick brown fox jumps over the lazy dog."
        tagged_sentence = pos_tagging(sentence)
        print(tagged_sentence)
        """
    ]


def stopwords():
    return [
        """
        import nltk
        from nltk.tokenize import word_tokenize
        from nltk.corpus import stopwords

        nltk.download('punkt', quiet=True)
        nltk.download('stopwords', quiet=True)
        """,
        """
        def remove_stopwords(corpus):
            stop_words = set(stopwords.words('english'))
            filtered_corpus = []

            for document in corpus:
                words = word_tokenize(document.lower())
                filtered_words = [word for word in words if word.isalnum() and word not in stop_words]
                filtered_corpus.append(" ".join(filtered_words))

            return filtered_corpus
        """,
        """
        dummy_corpus = [
            "This is the first document.",
            "This document is the second document.",
            "And this is the third one.",
            "Is this the first document?",
            "This is another document."
        ]
        """,
        """
        filtered_corpus = remove_stopwords(dummy_corpus)
        print(filtered_corpus)
        """
    ]


def smoothing():
    return [
        """
        import nltk
        from collections import Counter
        from nltk.tokenize import word_tokenize
        """,
        """
        nltk.download('punkt')
        """,
        """
        corpus = [
            'the', 'cat', 'sat', 'on', 'the', 'mat',
            'the', 'quick', 'brown', 'fox', 'jumps', 'over', 'the', 'lazy', 'dog'
        ]

        v = Counter(corpus)
        n = sum(v.values())

        print(f"Word count: \n{v}")
        print(f"Total count: {n}")
        """,
        """
        vocab = len(v)
        total_vocab = vocab+n   
        """,
        """
        # laplace smoothing

        probs = {}
        for word, count in v.items():
            probs[word] = (count + 1) / n

        print("\nProbabilities:")
        for key, value in probs.items():
            print(f"{key}, {round(value, 4)}")  
        """,
        """
        # Add-k Smoothing

        k=0.5
        additive_probs = {}
        for word, count in v.items():
            var = (count + k) / (n + k * vocab)
            additive_probs[word] = var

        print("\nProbabilities: ")
        for key, value in additive_probs.items():
            print(f"{key}, {round(value, 4)}")        
        """,
        """
        # preparing the data
        tokenized_corpus = []
        for sentence in corpus:
            var = sentence.lower()
            var = word_tokenize(sentence)
            tokenized_corpus.append(var)

        corpus_words = []
        for i in tokenized_corpus:
            for word in i:
                corpus_words.append(word)
            
        vocab = set(corpus_words)
        def grams(n):
            result = []
            for i in range(len(corpus_words)-n+1):
                result.append(tuple(corpus_words[i:i+n]))
            return result
        """,
        """
        unigram = grams(1)
        unigram_counter = Counter(unigram)
        bigrams = grams(2)
        bigrams_counter = Counter(bigrams)
        """,
        """
        unigram_prob = {unigram: count / n for unigram, count in unigram_counter.items()}
        bigram_prob = {bigram: count / n for bigram, count in bigrams_counter.items()}
        """,
        """
        # Interpolated smoothing

        l1 = 0.7
        l2 = 0.3

        interpolated_prob = {}
        for (w1, w2), count in bigrams_counter.items():
            bigram_p = bigram_prob.get((w1, w2), 0)
            unigram_p = unigram_prob.get(w2, 0)
            
            interpolated_prob[(w1, w2)] = l1 * bigram_p + l2 * unigram_p

        for key, value in interpolated_prob.items():
            print(f"{key}, {round(value, 4)}")
        """
    ]


def rule():
    return [
        """
        positive_words = ['good', 'happy', 'joyful', 'excellent', 'love', 'amazing', 'great', 'positive']
        negative_words = ['bad', 'sad', 'angry', 'horrible', 'hate', 'terrible', 'awful', 'negative']

        def sentiment_analysis(sentence):
            words = sentence.lower().split()

            positive_count = 0
            negative_count = 0

            for word in words:
                if word in positive_words:
                    positive_count += 1
                elif word in negative_words:
                    negative_count += 1

            if positive_count > negative_count:
                return "Positive Sentiment"
            elif negative_count > positive_count:
                return "Negative Sentiment"
            else:
                return "Neutral Sentiment"

        sentence = input("Enter a sentence: ")

        result = sentiment_analysis(sentence)
        print(f"Sentiment: {result}")
        """,
        """A Idiot wrote this code""",
        """Execute the first""",
        """
        nltk.download('punkt')
        nltk.download('vader_lexicon')

        import nltk
        from nltk.sentiment.vader import SentimentIntensityAnalyzer
        
        def rule_based(sentence):
            analyzer = SentimentIntensityAnalyzer()
            scores = analyzer.polarity_scores(sentence)
            if scores['compound'] >= 0.05:
                return 'Positive'
            elif series['compound'] <= -0.05:
                return 'Negative'
            return 'Neutral'
        sentence = "This view is not bad"
        sentiment = rule_based(sentence)
        print(sentiment)
        """
    ]